package com.example.afrishop_v3.controllers;

import com.example.afrishop_v3.base.Result;
import com.example.afrishop_v3.base.StateConstant;
import com.example.afrishop_v3.config.DomainProperties;
import com.example.afrishop_v3.enums.ResultCodeEnum;
import com.example.afrishop_v3.enums.SexEnum;
import com.example.afrishop_v3.enums.UserTypeEnum;
import com.example.afrishop_v3.models.Network;
import com.example.afrishop_v3.models.TbCfCoupon;
import com.example.afrishop_v3.models.TbCfToicoupon;
import com.example.afrishop_v3.models.TbCfUserInfo;
import com.example.afrishop_v3.payload.request.LoginRequest;
import com.example.afrishop_v3.payload.response.MessageResponse;
import com.example.afrishop_v3.repository.NetworkRepository;
import com.example.afrishop_v3.repository.TbCfCouponRepository;
import com.example.afrishop_v3.repository.TbCfToicouponRepository;
import com.example.afrishop_v3.repository.UserRepository;
import com.example.afrishop_v3.security.jwt.JwtUtils;
import com.example.afrishop_v3.security.services.UserDetailsImpl;
import com.example.afrishop_v3.util.IdUtil;
import org.hashids.Hashids;
import org.springframework.http.ResponseEntity;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.web.bind.annotation.*;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@CrossOrigin(origins = "*", maxAge = 3600)
@RestController
@RequestMapping("/api/auth")
public class AuthController extends Controller {
	private final AuthenticationManager authenticationManager;

	private final UserRepository userRepository;
	private final TbCfCouponRepository couponRepository;
	private final TbCfToicouponRepository toicouponRepository;
	private final NetworkRepository networkRepository;


	private final PasswordEncoder encoder;
	private final DomainProperties domainProperties;

	private final JwtUtils jwtUtils;

	public AuthController(AuthenticationManager authenticationManager, UserRepository userRepository, TbCfCouponRepository couponRepository, TbCfToicouponRepository toicouponRepository, NetworkRepository networkRepository, PasswordEncoder encoder, DomainProperties domainProperties, JwtUtils jwtUtils) {
		this.authenticationManager = authenticationManager;
		this.userRepository = userRepository;
		this.couponRepository = couponRepository;
		this.toicouponRepository = toicouponRepository;
		this.networkRepository = networkRepository;
		this.encoder = encoder;
		this.domainProperties = domainProperties;
		this.jwtUtils = jwtUtils;
	}

	@PostMapping("/signin")
	public Result authenticateUser(@RequestBody LoginRequest loginRequest) {
		Optional<TbCfUserInfo> byAccount = userRepository.findByFirebaseUid(loginRequest.getAccount());
		if( !byAccount.isPresent() ){
			return new Result<>(ResultCodeEnum.VALIDATE_ERROR.getCode(),"User not found");
		}


		Authentication authentication = authenticationManager.authenticate(
				new UsernamePasswordAuthenticationToken(loginRequest.getAccount(), loginRequest.getPassword()));

		SecurityContextHolder.getContext().setAuthentication(authentication);
		String jwt = jwtUtils.generateJwtToken(authentication);
		
		UserDetailsImpl userDetails = (UserDetailsImpl) authentication.getPrincipal();
		List<String> roles = userDetails.getAuthorities().stream()
				.map(GrantedAuthority::getAuthority)
				.collect(Collectors.toList());

		TbCfUserInfo userInfo = byAccount.get();

		userInfo.setToken(jwt);

		if( loginRequest.getFcm() != null ){
			userInfo.setFcm(loginRequest.getFcm());
			userRepository.save(userInfo);
		}

		if( userInfo.hasFcm() ){
			sendNotification(userInfo.getFcm(),"Welcome ",userInfo.display() + ", Welcome and start shopping !!!");
		}

		return new Result<>(userInfo);
	}

	@PostMapping("/signup")
	public Result<?> registerUser( @RequestBody TbCfUserInfo signUpRequest) {
		try {
			Optional<TbCfUserInfo> byAccount = userRepository.findByAccount(signUpRequest.getAccount());
			if ( byAccount.isPresent() ) {
				return new Result(ResultCodeEnum.VALIDATE_ERROR.getCode(),"Error: Username is already taken!");
			}

			Optional<TbCfUserInfo> byEmail = userRepository.findByFirebaseUid(signUpRequest.getEmail());

			if ( byEmail.isPresent() ) {
				return new Result(ResultCodeEnum.VALIDATE_ERROR.getCode(),"Error: Email is already in use!");
			}


			signUpRequest.setFirebaseUid(signUpRequest.getEmail());

			signUpRequest.setUserId(IdUtil.createIdbyUUID());

			fillUserNecessayInfo(signUpRequest);

			String password = signUpRequest.getPassword();

			signUpRequest.setPassword(encoder.encode(password));

			fixCode(signUpRequest);

			TbCfUserInfo userInfo = userRepository.save(signUpRequest);


			//addToNetwork(signUpRequest);


			fixCoupon(signUpRequest);

			return authenticateUser(new LoginRequest(userInfo.getFirebaseUid(),password,userInfo.getFcm()));
		}
		catch (Exception e){
			System.out.println(e.getMessage());
			return new Result<>(ResultCodeEnum.SERVICE_ERROR.getCode(),e.getMessage());
		}
	}





	@PostMapping(value = "/register/user")
	public Result checkFirebase(@RequestBody TbCfUserInfo user) throws ParseException {
		//Data to be userInfoVo
//        {
//            "firebaseUid":"firebaseUid",
//            "name":"name",
//            "email":"email",
//            "phone":"phone",
//            "birthDate":"date",
//            "token":"token",
//        }

		// Check if firebase token is valid
		boolean isTokenValid = user.getToken() != null && validateFirebaseToken(user.getToken());


		// if valid do sign in if firebase Uid exist in database or register as new user
		if (isTokenValid) {
			//Query to find user from database by firebase uid
			Optional<TbCfUserInfo> optional = userRepository.findByFirebaseUid(user.getFirebaseUid());
			if ( !optional.isPresent() ) {




				String userid = IdUtil.createIdbyUUID();

				user.setPassword(encoder.encode(user.getFirebaseUid()));
				user.setUserId(userid);

				fillUserNecessayInfo(user);
				user = userRepository.save(user);



				//赠送用户优惠券
				fixCoupon(user);
			} else {
				TbCfUserInfo userInfo = optional.get();

				if( user.hasFcm() ){
					userInfo.setFcm(user.getFcm());
				}

				String property = domainProperties.getProperty("user.avatar");

				property = property == null ? "" : property;

				boolean b = property.equals(userInfo.getAvatar()) || userInfo.getAvatar() == null || userInfo.getAvatar().isEmpty();


				if( user.getAvatar() != null && b){
					userInfo.setAvatar(user.getAvatar());
				}
				boolean b1 = userInfo.getNick() == null || userInfo.getNick().isEmpty();
				if( user.getNick() != null && b1){
					userInfo.setNick(user.getNick());
				}

				user = userInfo;

			}
			// generate token codes has been moved downwards from if condition of checking if user doesn't exist in database, because even if
			// user exist we have to generate token also
			if( user.getPassword() == null ){
				user.setPassword(encoder.encode(user.getFirebaseUid()));// Assign user from database to the user we have to return back to request
			}



			fixCode(user);

			//addToNetwork(user);

			user.setLastLoginTime(new Date());
			userRepository.save(user);
			//注册成功 创建token

			return authenticateUser(new LoginRequest(user.getFirebaseUid(),user.getFirebaseUid()));


		} else {
			return new Result<>(ResultCodeEnum.ILLEGAL_ARGUMENT.getCode(),ResultCodeEnum.ILLEGAL_ARGUMENT.getDesc());
		}

	}


	private void fixCoupon(TbCfUserInfo user){
		try{
			List<TbCfCoupon> couponVailList = couponRepository.findAllByCouponVaild(1);
			//获取当前时间的时分秒
			SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd 00:00:00");//设置日期格式
			Calendar c = Calendar.getInstance();
			c.add(Calendar.DATE, +7);
			Date time = c.getTime();
			Date startDate = sdf.parse(sdf.format(new Date()));
			Date endDate = sdf.parse(sdf.format(time));
			for (TbCfCoupon tbCfCoupon : couponVailList) {
				TbCfToicoupon toi = new TbCfToicoupon();
				//把上面获取到的值，赋值到实体类中
				toi.setToitableId(IdUtil.createIdbyUUID());
				toi.setCoupon(tbCfCoupon);
				toi.setUserId(user.getUserId());
				toi.setStartTime(startDate);
				toi.setIdentification(3);
				toi.setEnableFlag(1);
				toi.setEndTime(endDate);
				toicouponRepository.save(toi);
			}
		}catch (Exception e){

		}
	}

	private void fixCode(TbCfUserInfo user){
		if( user.getCode() == null){
			Hashids hashids = new Hashids("big father is daddy", 6);
			Long codeCount = user.getCodeCount();

			if( codeCount == null){
				codeCount = userRepository.count();
			}

			user.setCode(hashids.encode(codeCount));
		}
	}

	private void addToNetwork(TbCfUserInfo user){
		Network top = networkRepository.findTopByOrderByIdDesc();

		if( top != null && !user.invited() ) {
			if (!networkRepository.existsByUserInfo_UserIdAndNetworkInfo_UserId(top.getNetworkId(), user.getUserId())){
				Network network = new Network();
				network.setUserInfo(top.getNetworkInfo());
				network.setNetworkInfo(user);
				//network.setId(uid());
				networkRepository.save(network);
				user.setInvitation(1);
			}
		}
	}

	private void fillUserNecessayInfo(TbCfUserInfo tbCfUserInfoVo) {
		if( tbCfUserInfoVo.getAvatar() == null) tbCfUserInfoVo.setAvatar(domainProperties.getProperty("user.avatar"));
		tbCfUserInfoVo.setUserNo(IdUtil.createIdByDate());
		tbCfUserInfoVo.setPhoneFlag(StateConstant.INVALID);
		tbCfUserInfoVo.setLoginCount(0);
		tbCfUserInfoVo.setCreateTime(new Date());
		tbCfUserInfoVo.setSex(SexEnum.UNKNOW.getCode());
		tbCfUserInfoVo.setInvitedCount(0);
		tbCfUserInfoVo.setEnableFlag(StateConstant.VALID);
		//目前有验证码的都是邮箱类型
		tbCfUserInfoVo.setUserType(UserTypeEnum.FACEBOOK.getCode());
		tbCfUserInfoVo.setEmailFlag(StateConstant.INVALID);
	}
}
